# search/config.py

import typer
import os
from dotenv import load_dotenv
from enum import Enum


load_dotenv()
app_level = os.environ.get('APP_LEVEL')
# create typer object.
app = typer.Typer(help="[bold green]Easiest[/bold green] way to [bold yellow]find[/bold yellow], [bold]read[/bold], [bold green]create[/bold green], [bold cyan]write[/bold cyan], and [bold red]delete[/bold red] a file :file_folder:.", 
                add_completion=False,
                pretty_exceptions_show_locals=False, 
                pretty_exceptions_enable=True,
                rich_markup_mode='rich')

class FileTypes(Enum):
    text = 'text'
    python = 'python'
    javascript = 'javascript'
    typescript = 'typescript'
    go = 'go'
    php = 'php'
    bash = 'bash'
    css = 'css'
    html = 'html'
    rust = 'rust'




# search/npyscreen_app.py

import npyscreen
import curses
import pathlib
import os
from search.callbacks import _some_log

# write your custom code editor here 
class CodeEditorApp(npyscreen.NPSAppManaged):
    def onStart(self):
        self.appTheme = npyscreen.setTheme(CustomTheme)
        self.addForm("MAIN", CodeEditor, name="Code Editor V1")

# test creating custom theme
class CustomTheme(npyscreen.ThemeManager):
    default_colors = {
        'DEFAULT'     : 'CYAN_BLACK',
        'FORMDEFAULT' : 'YELLOW_BLACK',
        'NO_EDIT'     : 'GREEN_BLACK',
        'STANDOUT'    : 'CYAN_BLACK',
        'CURSOR'      : 'WHITE_BLACK',
        'CURSOR_INVERSE': 'BLACK_WHITE',
        'LABEL'       : 'GREEN_BLACK',
        'LABELBOLD'   : 'WHITE_BLACK',
        'CONTROL'     : 'GREEN_BLACK',
        'IMPORTANT'   : 'GREEN_BLACK',
        'SAFE'        : 'GREEN_BLACK',
        'WARNING'     : 'YELLOW_BLACK',
        'DANGER'      : 'RED_BLACK',
        'CRITICAL'    : 'BLACK_RED',
        'GOOD'        : 'GREEN_BLACK',
        'GOODHL'      : 'GREEN_BLACK',
        'VERYGOOD'    : 'BLACK_GREEN',
        'CAUTION'     : 'YELLOW_BLACK',
        'CAUTIONHL'   : 'BLACK_YELLOW',
    }

# define our action form
class CodeEditor(npyscreen.ActionForm):
    # define custom button text
    OK_BUTTON_TEXT = "SAVE"
    CANCEL_BUTTON_TEXT = "EXIT"

    def draw_title_and_help(self):
        if self.name:
            title = self.name[:(self.columns-4)]
            title = ' ' + str(title) + ' '
            if isinstance(title, bytes):
                title = title.decode('utf-8', 'replace')
            self.add_line(0,58, 
                title, 
                self.make_attributes_list(title, curses.A_BOLD),
                self.columns-4
                )

    def create(self):
        self.filename = self.add(
            npyscreen.TitleFilename,
            begin_entry_at=18,
            name="Filename      :",
            relx=3,
            rely=2,
            )
        self.path = self.add(
            npyscreen.TitleText,
            begin_entry_at=18,
            name="Folder Path   :",
            relx=3, 
            rely=4,
            editable=True
            )
        self.code = self.add(
            npyscreen.MultiLineEdit,
            relx=2, 
            rely=6,
            editable=True,
            scroll_exit=True,
            )
    
    # add method for condition where user pick 'SAVE' button
    def on_ok(self):
        # raise error if filename not include file type
        if self.filename.value.find(".") == -1:
            raise ValueError(f"'filename' needs file type at the end, file: {self.filename.value}")
        
        # define path
        curr_path = pathlib.Path(self.path.value)
        if curr_path.is_dir():
            real_path = os.path.join(curr_path, self.filename.value)
            if not self.filename.value or self.path.value:
                raise _some_log.error_log(ValueError, message="Filename and Path required!")
            else:
                if os.path.exists(real_path):
                    raise _some_log.error_log(FileExistsError, f"File exists: {real_path}")
                else:
                    if self.filename.value.endswith('.txt'):
                        with open(os.path.join(self.path.value, self.filename.value), "a+") as file:
                            file.seek(0)
                            is_content_exist = file.read()

                            if is_content_exist:
                                file.write("\n"+self.code.value)
                            else:
                                file.write(self.code.value)
                            file.close()
                    else:
                        with open(os.path.join(self.path.value, self.filename.value), 'a+') as file:
                            file.seek(0)
                            is_code_exist = file.read()
                            
                            if is_code_exist:
                                file.write("\n\n"+self.code.value)
                            else:
                                file.write(f"# {self.filename.value}\n\n"+self.code.value)
                            file.close()
        self.parentApp.setNextForm(None)
    
    # add method for condition where user pick 'EXIT' button
    def on_cancel(self):
        self.parentApp.setNextForm(None)



# search/callback.py

import pathlib
import os
import rich
import typer
import art
from rich.progress import Progress, SpinnerColumn, TextColumn
from search import __app_name__, __version__
from datetime import date
from search.logs import CustomLog
from termcolor import colored


def _code_example() -> str:
    output = """# create/main.py
                        
def main() -> None:
    pass
                        
if __name__ == '__main__':
    main()
    """
    return output

# define private instance for CustomLog class
_some_log = CustomLog(format_log='%(name)s | %(asctime)s %(levelname)s - %(message)s')

# create version callback function.
def version_callback(value: bool) -> None:
    if value:
        _some_log.info_log(message='Checking app version',)
        rich.print(f"[bold]{__app_name__} version[/bold]: {__version__}")
        raise typer.Exit()

# cretae info callback function.
def info_callback(value: bool) -> None:
    if value:
        _some_log.info_log(message='Checking app information')
        # show logo
        ascii_art = art.text2art("SEFILE", font="swampland", chr_ignore=True)
        print(f"\n{colored(ascii_art, color='green', attrs=['bold'])}\n")
        # create long text
        output = f"""[yellow]{'*'*40}|[bold]Information[/bold]|{'*'*40}[/yellow]\
                    \n\n[bold]App name[/bold]: {__app_name__}\
                    \n[bold]{__app_name__} version[/bold]: {__version__}\
                    \n\n[bold]Creator name[/bold]: Faisal Ramadhan\
                    \n[bold]Creator email[/bold]: faisalramadhan1299@gmail.com\
                    \n[bold]Creator github[/bold]: https://github.com/kolong-meja\
                """
        rich.print(output)
        raise typer.Exit()

def auto_create_callback(value: bool) -> None:
    if value:
        curr_path = os.path.join(pathlib.Path.home(), "Create")
        if os.path.exists(curr_path):
            raise _some_log.error_log(FileExistsError, f"Folder exists: {curr_path}")
        else:
            os.mkdir(curr_path)
            real_path = os.path.join(curr_path, 'main.py')
            if os.path.exists(real_path):
                raise _some_log.error_log(FileExistsError, f"File exists: {real_path}")
            else:
                with open(real_path, 'w+') as file:
                    file.write(_code_example())
                    _some_log.info_log(message="Auto create file")
                    rich.print(f"[bold green]Success creating file[/bold green], {real_path}")
                    raise typer.Exit()

def file_startswith(value: str) -> None:
    if value:
        # user home root.
        user_home_root = pathlib.Path.home()
        # scan all root from user home root.
        scanning_directory = os.walk(user_home_root, topdown=True)
        file_total = 0
        # iterate all directory.
        with Progress(
            SpinnerColumn(spinner_name="dots9"),
            TextColumn("[progress.description]{task.description}"),
            auto_refresh=True, 
            transient=True,
            get_time=None,
            ) as progress:
            task = progress.add_task(f"Find file startswith '{value}' from {user_home_root}", total=100_000_000)
            for root, dirs, files in scanning_directory:
                for file in files:
                    # filter file same as filename param.
                    if file.startswith(value):
                        file_total += 1 
                        # join the root and file.
                        root = f"[white]{root}[/white]"
                        file = f"[bold yellow]{file}[/bold yellow]"
                        fullpath = os.path.join(root, file)
                        rich.print(f"{fullpath}")
                        progress.advance(task)
        
        # do logging below,
        if file_total != 0:
            _some_log.info_log(message=f"Find '{value}' file with '--startswith' flag.")
            rich.print(f"Search file startswith '{value}' [bold green]success![/bold green]")
            raise typer.Exit()
        else:
            _some_log.error_log(FileNotFoundError, message=f"File '{value}' not found.")

def file_endswith(value: str) -> None:
    if value:
        # user home root.
        user_home_root = pathlib.Path.home()
        # scan all root from user home root.
        scanning_directory = os.walk(user_home_root, topdown=True)
        file_total = 0
        with Progress(
            SpinnerColumn(spinner_name="dots9"),
            TextColumn("[progress.description]{task.description}"),
            auto_refresh=True, 
            transient=True,
            ) as progress:
            task = progress.add_task(f"Find file startswith '{value}' from {user_home_root}", total=100_000_000)
            # iterate all directory.
            for root, dirs, files in scanning_directory:
                for file in files:
                    # filter file same as filename param.
                    if file.endswith(value):
                        file_total += 1
                        root = f"[white]{root}[/white]"
                        file = f"[bold yellow]{file}[/bold yellow]"
                        # join the root and file.
                        fullpath = os.path.join(root, file)
                        rich.print(f"{fullpath}")
                        progress.advance(task)
        
        # do logging below,
        if file_total != 0:
            _some_log.info_log(message=f"Find '{value}' file with '--endswith' flag.")
            rich.print(f"Search file endswith '{value}' [bold green]success![/bold green]")
            raise typer.Exit()
        else:
            _some_log.error_log(FileNotFoundError, message=f"File '{value}' not found.")
            


# search_app/command.py

import os
import pathlib
import typer
from search.controllers import (
    find_logic,
    create_logic,
    read_logic,
    write_logic,
    delete_logic,
    )
from search.callbacks import (
    info_callback,
    file_startswith,
    file_endswith,
    version_callback,
    auto_create_callback,
    )
from search.config import (
    app, 
    FileTypes
    )
from typing import Optional


@app.command(help="Command to [bold yellow]find[/bold yellow] a file by it's name 🔍.")
def find(filename: str = typer.Argument(help="Name of file to [bold yellow]search[/bold yellow] for. :page_facing_up:", 
                                        metavar="FILENAME"),
        path: str = typer.Argument(metavar="PATH", 
                                   help="Directory path for the file to search."),
        startswith: str = typer.Option(default=None, 
                                        help="Search specific file with 'startswith' method :rocket:.", 
                                        is_eager=True, 
                                        callback=file_startswith), 
        endswith: str = typer.Option(default=None, 
                                    help="""Search specific file with 
                                    'endswith' method :sunrise_over_mountains:""", 
                                    is_eager=True, 
                                    callback=file_endswith)) -> None:
    """
    TODO: Define find logic from controllers
    """
    find_logic(filename=filename, path=path, startswith=startswith, endswith=endswith)

@app.command(help="Command to [bold green]create[/bold green] new file followed by a path :cookie:.")
def create(filename: str = typer.Argument(default=None, metavar="FILENAME", 
                                          help="Name of file to [bold green]create[/bold green] a new one. :page_facing_up:"),
           path: str = typer.Argument(default=None, metavar="PATH", 
                                      help="Directory [bold blue]path[/bold blue] for file that has been created. :file_folder:"),
           auto: Optional[bool] = typer.Option(None, "--auto", 
                                               help=f"Automatically create main.py file in {os.path.join(pathlib.Path.home(), 'Create')}", 
                                               is_eager=True,
                                               callback=auto_create_callback)) -> None:
    """
    TODO: Define create logic from controllers
    """
    create_logic(filename=filename, path=path, auto=auto)
@app.command(help="Command to [bold]read[/bold] a file from a directory :book:.")
def read(filename: str = typer.Argument(metavar="FILENAME", 
                                        help="Name of file to read of. :page_facing_up:"),
        path: str = typer.Argument(metavar="PATH", 
                                   help="Directory path of file that want to read of. :file_folder:"),
        read_type: FileTypes = typer.Option(default=FileTypes.text, 
                                       is_flag=True, 
                                       help="Read files according to type selection. :computer:")) -> None:
    """
    TODO: Define read logic from controllers
    """
    read_logic(filename=filename, path=path, read_type=read_type)

@app.command(help="Command to [bold blue]write[/bold blue] one file :page_facing_up:")
def write() -> None:
    """
    TODO: Define write logic from controllers
    """
    write_logic()

@app.command(help="Command to [bold red]delete[/bold red] one or more file :eyes:.")
def delete(filename: str = typer.Argument(metavar="FILENAME", 
                                        help="Name of file to be deleted. :page_facing_up:"),
            path: str = typer.Argument(metavar="PATH", 
                                       help="Directory of file to be deleted. :file_folder:")) -> None:
    """
    TODO: Define write logic from controllers
    """
    delete_logic(filename=filename, path=path)

# main function in here!
@app.callback()
def main(version: Optional[bool] = typer.Option(None, "--version", "-v", 
                                                help="Show version of search CLI.", 
                                                is_eager=True, 
                                                callback=version_callback),
         info: Optional[bool] = typer.Option(None, "--info", "-i", 
                                             help="Display info about the application", 
                                             is_eager=True, 
                                             callback=info_callback)) -> None: return






# search_app/app.py

import platform
from search import command, __app_name__


def main() -> None:
    if platform.uname().system != "Linux":
        raise Exception(f"Program not supported yet for '{platform.uname().system}' system.")
    else:
        command.app(prog_name=__app_name__)

if __name__ == "__main__":
    main()


# search/logs.py

import os
import pathlib
import logging


def log_file():
    fullpath = os.path.join(pathlib.Path.cwd(), 'logs')
    has_dir = os.path.isdir(fullpath)
    
    if has_dir:
        file_target = os.path.join(fullpath, 'log.txt')
        return file_target
    else:
        pathlib.Path(fullpath).mkdir(exist_ok=False)
        file_target = os.path.join(fullpath, 'log.txt')
        return file_target

# create factory for exception.
def exception_factory(exception, message: str) -> Exception:
    return exception(message)

class CustomLog:
    def __init__(
            self,
            format_log: str = '%(name)s | %(asctime)s %(levelname)s - %(message)s'
            ) -> None:
        self.format_log = format_log

    def info_log(self, message: str) -> None:
        logging.basicConfig(filename=log_file(), filemode='a+', 
                            format=self.format_log,
                            level=logging.INFO)
        logging.info(message)
    
    def error_log(self, exception, message: str) -> None:
        logging.basicConfig(filename=log_file(), filemode='a+', 
                            format=self.format_log,
                            level=logging.ERROR)
        logging.error(message)
        raise exception_factory(exception, message)
    
    def debug_log(self, message: str) -> None:
        logging.basicConfig(filename=log_file(), filemode='a+', 
                            format=self.format_log,
                            level=logging.DEBUG)
        logging.error(message)
    
    def warning_log(self, message: str) -> None:
        logging.basicConfig(filename=log_file(), filemode='a+', 
                            format=self.format_log,
                            level=logging.WARNING)
        logging.error(message)
    
    def critical_log(self, message: str) -> None:
        logging.basicConfig(filename=log_file(), filemode='a+', 
                            format=self.format_log,
                            level=logging.CRITICAL)
        logging.error(message)
    
    def notset_log(self, message: str) -> None:
        logging.basicConfig(filename=log_file(), filemode='a+', 
                            format=self.format_log,
                            level=logging.NOTSET)
        logging.error(message)

